function Start-ExWindowsService	{
	#Requires -Version 2.0
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		# Name of computer on which to start service(s)
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $ComputerName = $env:ComputerName,

		# Name of specific service to start
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $Name
	)
	if (!($Name)){
		$Services = Get-WmiObject -Class Win32_Service -ComputerName $ComputerName | Where-Object {$_.Name -match "^MSExchange" -and $_.State -ne "Running" -and $_.StartMode -eq "Auto"} | Sort-Object name
	}else{
		$Services = Get-Service -InputObject (Get-Service -ComputerName $ComputerName -Name $Name)
	}
	ForEach ($Service in $Services) {
		$ThisName = $Service.Name
		$ThisDisplayname = $Service.DisplayName
		do {Write-Progress -Activity "Start-CsWindowsService" -CurrentOperation "$ThisDisplayName ($ThisName)" -Status "Start-CsWindowsService";start-sleep -s 1}while(Start-Service -InputObject (Get-Service -ComputerName $ComputerName -Name $Service.Name) -WarningAction SilentlyContinue)
	}
} # end function Start-ExWindowsService

function Stop-ExWindowsService	{
	#Requires -Version 2.0
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		# Specifies that all services should be forcefully stopped
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
		[switch] $Force,

		# Name of computer on which to stop service(s)
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $ComputerName = $env:ComputerName,

		# Name of specific service to stop
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $Name
	)
	if (!($Name)){
		$Services = Get-WmiObject -Class Win32_Service -ComputerName $ComputerName | Where-Object {$_.Name -match "^MSExchange" -and ($_.Name -ne "MSExchangeADTopology") -and ($_.State -eq "Running")} | Sort-Object name
	}else{
		$Services = Get-Service -InputObject (Get-Service -ComputerName $ComputerName -Name $Name)
	}
	if ($force){
		ForEach ($Service in $Services) {
			$ThisName = $Service.Name
			$ThisDisplayname = $Service.DisplayName
			do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "$ThisDisplayName ($ThisName)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -InputObject (Get-Service -ComputerName $ComputerName -Name $Service.Name) -force -WarningAction SilentlyContinue)
		}
		do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "Microsoft Exchange Active Directory Topology (MSExchangeADTopology)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -name "MSExchangeADTopology" -Force -WarningAction SilentlyContinue)
		Start-Sleep -s 3
		# AD Topology service will sometimes restart by itself. Let's stop it again
		do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "Microsoft Exchange Active Directory Topology (MSExchangeADTopology)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -name "MSExchangeADTopology" -Force)
	} else {
		ForEach ($Service in $Services) {
			$ThisName = $Service.Name
			$ThisDisplayname = $Service.DisplayName
			do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "$ThisDisplayName ($ThisName)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -InputObject (Get-Service -ComputerName $ComputerName -Name $Service.Name) -WarningAction SilentlyContinue)
		}
		do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "Microsoft Exchange Active Directory Topology (MSExchangeADTopology)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -name "MSExchangeADTopology" -WarningAction SilentlyContinue)
		Start-Sleep -s 3
		# AD Topology service will sometimes restart by itself. Let's stop it again
		do {Write-Progress -Activity "Stop-CsWindowsService" -CurrentOperation "Microsoft Exchange Active Directory Topology (MSExchangeADTopology)" -Status "Stop-CsWindowsService";start-sleep -s 1}while(Stop-Service -name "MSExchangeADTopology" -WarningAction SilentlyContinue)
	}
} # end function Stop-ExWindowsService

function Get-ExWindowsService	{
	#Requires -Version 2.0
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		# Name of computer on which to query service(s)
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $ComputerName = $env:ComputerName,

		# Name of specific service to query
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $Name
	)
	if (!($Name)){
		return Get-WmiObject -Class Win32_Service -ComputerName $ComputerName | Where-Object {$_.Name -match "^MSExchange"} | Sort-Object name | Select-Object Name, StartMode, State
	}else{
		return Get-WmiObject -Class Win32_Service -ComputerName $ComputerName | Where-Object {$_.Name -match "$Name"} | Sort-Object name | Select-Object Name, StartMode, State
	}
} # end function Get-ExWindowsService

function Set-ExWindowsServiceRecoveryOption	{
	#Requires -Version 3.0
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
	# Name of computer on which to configure service(s)
	[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
	[ValidateNotNullOrEmpty()]
	[string] $ComputerName = $env:ComputerName
	)
	$services = Get-WMIObject Win32_Service -ComputerName $ComputerName | Where-Object {$_.Name -match "^MSExchange" -and $_.startmode -eq "Auto"}; foreach ($service in $services){sc.exe failure $service.name reset= 86400 actions= restart/5000/restart/5000/reboot/5000}
}